﻿# ------------------------------------------------------------
# Copyright (c) CLOUD ASSERT LLC.  All rights reserved.
# ------------------------------------------------------------

# Caution: Do not rename the following param
param($inputParam)

Add-Type -Path 'C:\Program Files\CloudAssertBilling\CloudAssert.WAP.Billing.AgentService\CloudAssert.Billing.dll'

function ConnectToVBRServer {
    Add-PSSnapin VeeamPSSnapin
    DisconnectVBRServer
    $vbrServerName = 'VbrServerName'
    $vbrUserName = 'CloudAssert'
    $vbrPassword = 'password'
    Connect-VBRServer -Server $vbrServerName -User $vbrUserName -Password $vbrPassword
}

function DisconnectVBRServer {
    Disconnect-VBRServer
}

function GetProperties{
param($re, $isTenant)

    $props = New-Object 'System.Collections.Generic.Dictionary[string,string]'

    if($isTenant) {
        $props.Add('TenantName', $re.Name)
        $props.Add('TenantId', $re.Id)
    }
    else {
        $props.Add('RepositoryName', $re.RepositoryFriendlyName)
        $props.Add('ResourceId', $re.Id)
        $props.Add('RepositoryQuota', $re.RepositoryQuota)
        $props.Add('RepositoryQuotaPath', $re.RepositoryQuotaPath)
        $props.Add('WanAccelerationEnabled',$re.WanAccelerationEnabled)
        $props.Add('WanAccelerator', $re.WanAccelerator)
    }

    return $props
}

function GetUsageRecord{
param($wapSubId, $props, $resources)

    $utcTime = (Get-Date).ToUniversalTime()
    $startTime = [datetime](Get-Date $utcTime -Format 'yyyy-MM-dd HH:00')
    $endTime = $startTime.AddHours(1)

    $r = New-Object 'CloudAssert.Billing.Contracts.UsageRecord'
    $r.EventId = -1
    $r.ProviderName = "veeam"
    $r.ExternalRecordId = "-1"
    $r.SubscriptionId = $wapSubId
    $r.ServiceType = "CloudConnect"
    $r.StartTime = $startTime
    $r.EndTime = $endTime
    $r.Properties = $props
    $r.Resources = $resources
    return $r
}

function ProcessResourceOfVacTenant {
param($re, $wapSubId)
    
    $props = GetProperties $re $false

    $resources = New-Object 'System.Collections.Generic.Dictionary[string,string]'
    $resources.Add('UsedSpace', $re.UsedSpace)
    $record = GetUsageRecord $wapSubId $props $resources
    $record.ResourceId = $re.Id

    return $record
}

function ProcessVacTenant {
param($vacTenant, $wapSubId)
    $vacTenantResources = $vacTenant.Resources
    $private:records = New-Object 'CloudAssert.Billing.Contracts.UsageRecordList'

    $props = GetProperties $vacTenant $true

    $resources = New-Object 'System.Collections.Generic.Dictionary[string,string]'
    $resources.Add('VMCount', $vacTenant.VMCount)
    $resources.Add('ReplicaCount', $vacTenant.ReplicaCount)
    $resources.Add('WorkstationCount', $vacTenant.WorkstationCount)
    $resources.Add('ServerCount', $vacTenant.ServerCount)

    $private:records.Add($(GetUsageRecord $wapSubId $props $resources))

    foreach($re in $vacTenantResources){
        $record = ProcessResourceOfVacTenant $re $wapSubId
        if ($record -ne $null) {
            $private:records.Add($record)
        }
    }

    return ,$private:records
}

function GenerateRecords {
param($inputDetails)

    $processedIds = New-Object 'System.Collections.Generic.List[string]'
    $erroredIds = New-Object 'System.Collections.Generic.List[string]'
    $errorMessages = New-Object 'System.Collections.Generic.List[string]'
    $recordsList = New-Object 'CloudAssert.Billing.Contracts.UsageRecordList'

    $vacTenants = Get-VBRCloudTenant

    foreach ($kvp in $inputDetails.VacSubscriptionIds.GetEnumerator()){
        try {
            $filter = {$_.Name -eq $kvp.Value}
            $vacTenant =  $vacTenants | Where $filter
            if($vacTenant -ne $null){
                $records = ProcessVacTenant $vacTenant $kvp.Key
                if ($records -ne $null) {
                    $recordsList.AddRange($records)
                }
                $processedIds.Add($kvp.Key)
            }
        }
        catch {
            $errorMessage = $_.Exception.Message
            $erroredIds.Add($kvp.Key)
            $errorMessages.Add($errorMessage)
        }
    }

    $result = New-Object 'CloudAssert.Billing.UsageExtensions.UsageCollectionResult'
    $result.IsSuccess = $true
    $result.ProcessedIds = $processedIds
    $result.ErroredIds = $erroredIds
    $result.ErrorMessages = $errorMessages
    $result.Records = $recordsList
    $result.CallAfterSeconds = -1
    $result.AdditionalDetailsJson = ''

    $resultJson = ConvertTo-Json -Depth 10 $result
    return $resultJson
}

ConnectToVBRServer
GenerateRecords $inputParam
DisconnectVBRServer